@@ -34,7 +34,7 @@
 
 
 typedef int (*env_create_FN)(vps_handler *h, envid_t veid, int wait_p,
-	int err_p, void *data);
+				int old_wait_p, int err_p, void *data);
 
 /** Stop modes.
  */
@@ -102,7 +102,8 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
  */
 int vz_chroot(const char *root);
 int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
-	int wait_p[2], int err_p[2], env_create_FN fn, void *data);
+	int wait_p[2], int old_wait_p[2], int err_p[2],
+				env_create_FN fn, void *data);
 int vz_setluid(envid_t veid);
 int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags);
 #endif
