@@ -36,6 +36,7 @@
 
 #include "logger.h"
 #include "list.h"
+#include "bitmap.h"
 #include "config.h"
 #include "vzctl_param.h"
 #include "vzerror.h"
@@ -118,6 +119,7 @@ static vps_config config[] = {
 {"CPUUWEIGHT",	NULL, PARAM_CPUWEIGHT},
 {"CPULIMIT",	NULL, PARAM_CPULIMIT},
 {"CPUS",	NULL, PARAM_VCPUS},
+{"CPUMASK",	NULL, PARAM_CPUMASK},
 /* create param	*/
 {"ONBOOT",	NULL, PARAM_ONBOOT},
 {"CONFIGFILE",	NULL, PARAM_CONFIG},
@@ -142,7 +144,8 @@ static vps_config config[] = {
 {NULL,		NULL, -1}
 };
 
-const vps_config *conf_get_by_name(const vps_config *conf, const char *name)
+static const vps_config *conf_get_by_name(const vps_config *conf,
+		const char *name)
 {
 	const vps_config *p;
 
@@ -158,7 +161,7 @@ const vps_config *conf_get_by_name(const vps_config *conf, const char *name)
 	return NULL;
 }
 
-const vps_config *conf_get_by_id(const vps_config *conf, int id)
+static const vps_config *conf_get_by_id(const vps_config *conf, int id)
 {
 	const vps_config *p;
 
@@ -170,7 +173,7 @@ const vps_config *conf_get_by_id(const vps_config *conf, int id)
 	return NULL;
 }
 
-int opt_get_by_id(struct option *opt, int id)
+static int opt_get_by_id(struct option *opt, int id)
 {
 	struct option *p;
 
@@ -252,6 +255,28 @@ static int conf_parse_ulong(unsigned long **dst, const char *valstr)
 	return 0;
 }
 
+static int conf_parse_bitmap(unsigned long **dst, int nmaskbits,
+			     const char *valstr)
+{
+	if (*dst != NULL)
+		return ERR_DUP;
+
+	*dst = alloc_bitmap(nmaskbits);
+	if (*dst == NULL)
+		return ERR_NOMEM;
+
+	if (!strcmp(valstr, "all")) {
+		bitmap_set_all(*dst, nmaskbits);
+		return 0;
+	}
+	if (bitmap_parse(valstr, *dst, nmaskbits) != 0) {
+		free(*dst);
+		*dst = NULL;
+		return ERR_INVAL;
+	}
+	return 0;
+}
+
 static int conf_store_strlist(list_head_t *conf, char *name, list_head_t *val,
 		int allow_empty)
 {
@@ -303,6 +328,32 @@ static int conf_store_ulong(list_head_t *conf, char *name, unsigned long *val)
 	return conf_store_str(conf, name, buf);
 }
 
+static int conf_store_bitmap(list_head_t *conf, char *name,
+			     unsigned long *val, int nmaskbits)
+{
+	int ret;
+	char *buf;
+	unsigned int buflen = nmaskbits * 2;
+
+	if (val == NULL)
+		return 0;
+
+	if (bitmap_find_first_zero_bit(val, nmaskbits) == nmaskbits) {
+		conf_store_str(conf, name, "all");
+		return 0;
+	}
+
+	buf = malloc(buflen);
+	if (buf == NULL)
+		return ERR_NOMEM;
+
+	bitmap_snprintf(buf, buflen, val, nmaskbits);
+	ret = conf_store_str(conf, name, buf);
+
+	free(buf);
+	return ret;
+}
+
 /******************** Features *************************/
 static int parse_features(env_param_t *env, char *val)
 {
@@ -337,7 +388,7 @@ static void store_features(unsigned long long mask, unsigned long long known,
 	add_str_param(conf_h, buf);
 }
 
-int parse_ioprio(int id, io_param *io, char *val)
+static int parse_ioprio(int id, io_param *io, char *val)
 {
 	if (parse_int(val, &io->ioprio))
 		return ERR_INVAL;
@@ -504,7 +555,7 @@ static int parse_twoul_sfx(const char *str, unsigned long *val, int divisor)
 }
 
 /******************** totalmem *************************/
-int parse_meminfo(meminfo_param *param, const char *val)
+static int parse_meminfo(meminfo_param *param, const char *val)
 {
 	int mode;
 	char mode_nm[32];
@@ -553,7 +604,7 @@ static int store_meminfo(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	return 0;
 }
 
-int parse_ub(vps_param *vps_p, const char *val, int id, int divisor)
+static int parse_ub(vps_param *vps_p, const char *val, int id, int divisor)
 {
 	int ret;
 	ub_res res;
@@ -672,7 +723,7 @@ static int store_cap(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 }
 /********************** Network ************************/
 
-int check_ip_dot(char *ip)
+static int check_ip_dot(char *ip)
 {
 	int i;
 	char *str = ip;
@@ -768,7 +819,7 @@ static int check_netdev(const char *devname)
 	return 0;
 }
 
-int add_netdev(net_param *net, char *val)
+static int add_netdev(net_param *net, char *val)
 {
 	char *token;
 
@@ -1251,6 +1302,16 @@ static int parse_cpulimit(unsigned long **param, const char *str)
 	return 0;
 }
 
+static inline int parse_cpumask(cpumask_t **dst, const char *str)
+{
+	return conf_parse_bitmap((unsigned long **)dst, CPUMASK_NBITS, str);
+}
+
+static inline int store_cpumask(list_head_t *conf, char *name, cpumask_t *val)
+{
+	return conf_store_bitmap(conf, name, cpumask_bits(val), CPUMASK_NBITS);
+}
+
 static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
@@ -1269,6 +1330,9 @@ static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	case PARAM_VCPUS:
 		conf_store_ulong(conf_h, conf->name, cpu->vcpus);
 		break;
+	case PARAM_CPUMASK:
+		store_cpumask(conf_h, conf->name, cpu->mask);
+		break;
 	}
 	return 0;
 }
@@ -1290,7 +1354,8 @@ static int parse_mac_filter_cmd (veth_dev *dev, char *str)
 	return 0;
 }
 
-void generate_veth_name(int veid, char *dev_name_ve,  char *dev_name, int len)
+static void generate_veth_name(int veid, char *dev_name_ve,
+		char *dev_name, int len)
 {
 	char *name;
 	int id = 0;
@@ -1934,6 +1999,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_VCPUS:
 		ret = conf_parse_ulong(&vps_p->res.cpu.vcpus, val);
 		break;
+	case PARAM_CPUMASK:
+		ret = parse_cpumask(&vps_p->res.cpu.mask, val);
+		break;
 	case PARAM_MEMINFO:
 		ret = parse_meminfo(&vps_p->res.meminfo, val);
 		break;
@@ -2236,32 +2304,6 @@ int vps_save_config(envid_t veid, char *path, vps_param *new_p,
 	return ret;
 }
 
-int vps_remove_cfg_param(envid_t veid, char *path, char *name)
-{
-	list_head_t conf;
-	conf_struct *line;
-	int ret, found;
-
-	list_head_init(&conf);
-	if ((ret = read_conf(path, &conf)))
-		return ret;
-	if (list_empty(&conf))
-		return 0;
-	found = 0;
-	while ((line = find_conf_line(&conf, name, '=')) != NULL) {
-		free(line->val);
-		list_del(&line->list);
-		free(line);
-		found++;
-	}
-	if (found)
-		ret = write_conf(path, &conf);
-	free_str_param(&conf);
-
-	return ret;
-
-}
-
 /********************************************************************/
 int vps_parse_opt(envid_t veid, struct option *opts, vps_param *param,
 	int opt, char *rval, struct mod_action *action)
@@ -2463,6 +2505,7 @@ static void free_cpu(cpu_param *cpu)
 	FREE_P(cpu->weight)
 	FREE_P(cpu->limit)
 	FREE_P(cpu->vcpus)
+	FREE_P(cpu->mask);
 }
 
 static void free_dq(dq_param *dq)
@@ -2578,6 +2621,7 @@ static void merge_cpu(cpu_param *dst, cpu_param *src)
 	MERGE_P(weight)
 	MERGE_P(limit)
 	MERGE_P(vcpus)
+	MERGE_P(mask);
 }
 
 #define	MERGE_LIST(x)							\
